// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import { generateKeyPairSync } from 'crypto';
import utils from './utils';

/** @param {number} size */
export function randomPrimeRSA(size = 2048) {
    return typeof window !== 'undefined' ? getPrimeBrowser(size) : getPrimeJs(size);
}

/** @param {number} size */
async function getPrimeBrowser(size) {
    var key = await window.crypto.subtle.generateKey({
        name: "RSA-PSS",
        modulusLength: size,
        publicExponent: new Uint8Array([1, 0, 1]),
        hash: "SHA-256",
    }, true, ["sign", "verify"]);

    const { p, q, n } = await window.crypto.subtle.exportKey("jwk", key.privateKey);
    var pNum = BigInt.fromArray(Array.from(Buffer.from(p, "base64")), 256, false);
    var qNum = BigInt.fromArray(Array.from(Buffer.from(q, "base64")), 256, false);
    var nNum = BigInt.fromArray(Array.from(Buffer.from(n, "base64")), 256, false);

    return { n: nNum, q: qNum, p: pNum };
}

/**
 * @param {number} size 
 * @returns {Promise<{n: BigInt.BigInteger; q: BigInt.BigInteger; p: BigInt.BigInteger;}>}
 */
async function getPrimeJs(size) {
    var halfBytes = (size / 16) | 0
    var [p, q] = await Promise.all([ utils.randomPrimeBase(halfBytes), utils.randomPrimeBase(halfBytes) ]);
    
    return { q: q, p: p, n: p.times(q),  };
}

/**
 * @param {number} size 
 * @returns {Promise<{n: BigInt.BigInteger; q: BigInt.BigInteger; p: BigInt.BigInteger;}>}
 */
async function getPrimeNode(size) {
    throw new Error('Method not implemented');

    var { privateKey } = await generateKeyPairSync('rsa', {
        modulusLength: size,
        publicKeyEncoding: { type: 'spki', format: 'der' },
        privateKeyEncoding: { type: 'pkcs8', format: 'der' }
    });
}
