import assert from 'assert';
import C25519Key from '../src/c25519Key.js';
import AesSherableKey from '../src/aesSherableKey.js';
import C25519Cipher from '../src/c25519Cipher.js';

describe('C25519Key', function () {
    it('should serialize and deserialize', function () {
        for (let i = 0; i < 20; i++) {
            var key1 = C25519Key.generate();
            var key2 = C25519Key.from(key1.toString())

            assert.equal(key1.toString(), key2.toString());
        }
    });

    it('should encrypt and decrypt bytes', function () {
        for (let i = 0; i < 20; i++) {
            var key = C25519Key.generate();
            var m = new AesSherableKey();

            var cypher = key.encryptKey(m);
            var mTag = key.decryptKey(cypher.toArray());

            assert.equal(m.toString(), mTag.toString());
        }
    });

    it('should threshold decrypt aes key', function () {
        var threshold = 2;
        var ids = [1, 2, 3];

        for (let i = 0; i < 20; i++) {
            var key = C25519Key.generate();
            var shares = key.share(threshold, ids, true);
            var m = new AesSherableKey();

            var cipher = key.encryptKey(m);
            var cipherShared = shares.map(shr => shr.partialDecrypt(cipher.toArray()));
            var plain = C25519Cipher.decryptShares(cipherShared, ids);
            var mTag = AesSherableKey.from(plain);

            assert.equal(m.toString(), mTag.toString());
        }
    });
});
